@@ -4,7 +4,7 @@ NULL
 #' 
 #' @description Provides solution for enhancing the workflow of clustering
 #'   analyses and ggplot2-based elegant data visualization. Read more: 
-#'  \href{http://www.sthda.com/english/wiki/visual-enhancement-of-clustering-analysis-unsupervised-machine-learning}{Visual enhancement of clustering analysis}.
+#'  \href{https://www.datanovia.com/en/blog/cluster-analysis-in-r-simplified-and-enhanced/}{Visual enhancement of clustering analysis}.
 #' @param x numeric vector, data matrix or data frame
 #' @param FUNcluster a clustering function including "kmeans", "pam", "clara", 
 #'   "fanny", "hclust", "agnes" and "diana". Abbreviation is allowed.
@@ -58,7 +58,8 @@ NULL
 #' df <- scale(USArrests)
 #' 
 #' # Enhanced k-means clustering
-#' res.km <- eclust(df, "kmeans", nboot = 10)
+#' # nboot >= 500 is recommended
+#' res.km <- eclust(df, "kmeans", nboot = 2)
 #' # Silhouette plot
 #' fviz_silhouette(res.km)
 #' # Optimal number of clusters using gap statistics
@@ -66,10 +67,12 @@ NULL
 #' # Print result
 #'  res.km
 #'  
+#' \dontrun{
 #'  # Enhanced hierarchical clustering
-#'  res.hc <- eclust(df, "hclust", nboot = 10) # compute hclust
+#'  res.hc <- eclust(df, "hclust", nboot = 2) # compute hclust
 #'   fviz_dend(res.hc) # dendrogam
 #'   fviz_silhouette(res.hc) # silhouette plot
+#'}
 #'  
 #' @name eclust
 #' @rdname eclust
@@ -82,6 +85,15 @@ eclust <- function(x, FUNcluster = c("kmeans", "pam", "clara", "fanny", "hclust"
                    nboot = 100, verbose = interactive(),
                    seed = 123,  ...)
   {
+  has_seed <- exists(".Random.seed", envir = .GlobalEnv, inherits = FALSE)
+  if(has_seed) old_seed <- get(".Random.seed", envir = .GlobalEnv)
+  on.exit({
+    if(!has_seed && exists(".Random.seed", envir = .GlobalEnv, inherits = FALSE)){
+      rm(".Random.seed", envir = .GlobalEnv)
+    } else if(has_seed){
+      assign(".Random.seed", old_seed, envir = .GlobalEnv)
+    }
+  }, add = TRUE)
   set.seed(seed)
   data <- x
   if(stand) x <- scale(x)
@@ -164,4 +176,3 @@ eclust <- function(x, FUNcluster = c("kmeans", "pam", "clara", "fanny", "hclust"
 }
 
 
-
