@@ -95,13 +95,11 @@ fviz_dend <- function(x, k = NULL, h = NULL, k_colors = NULL, palette = NULL,  s
                       sub = NULL, ggtheme = theme_classic(),  ...)
 {
   
-#  if(.is_col_palette(k_colors)) palette <- k_colors
- # else palette <- NULL
-  if(missing(k_colors) & !is.null(palette)) {
+  if(missing(k_colors) && !is.null(palette)) {
     k_colors <- palette
     palette <- NULL
   }
-  if(!color_labels_by_k & is.null(label_cols)) label_cols <- "black"
+  if(!color_labels_by_k && is.null(label_cols)) label_cols <- "black"
   type <- match.arg(type)
   circular <- type == "circular"
   phylogenic <- type == "phylogenic"
@@ -134,7 +132,7 @@ fviz_dend <- function(x, k = NULL, h = NULL, k_colors = NULL, palette = NULL,  s
   else stop("Can't handle an object of class ", paste(class(x), collapse =", ") )
   if(is.null(method)) method <- ""
   else if(is.na(method)) method <- ""
-  if(is.null(sub) & method!="") sub = paste0("Method: ", method)
+  if(is.null(sub) && method != "") sub = paste0("Method: ", method)
   
   if(!is.null(dendextend::labels_cex(dend))) cex <- dendextend::labels_cex(dend)
   dend <- dendextend::set(dend, "labels_cex", cex) 
@@ -142,8 +140,8 @@ fviz_dend <- function(x, k = NULL, h = NULL, k_colors = NULL, palette = NULL,  s
   
   k <- .get_k(dend, k, h)
   if(!is.null(k)) {
-    if(ggpubr:::.is_col_palette(k_colors)) k_colors <- ggpubr:::.get_pal(k_colors, k = k)
-    else if(is.null(k_colors)) k_colors <- ggpubr:::.get_pal("default", k = k)
+    if(.is_color_palette(k_colors)) k_colors <- ggpubr::get_palette(k_colors, k = k)
+    else if(is.null(k_colors)) k_colors <- ggpubr::get_palette("default", k = k)
     dend <- dendextend::set(dend, what = "branches_k_color", k = k, value = k_colors)
     if(color_labels_by_k) dend <- dendextend::set(dend, "labels_col",  k = k, value = k_colors)
   }
@@ -163,7 +161,7 @@ fviz_dend <- function(x, k = NULL, h = NULL, k_colors = NULL, palette = NULL,  s
   if(max_height < 1) offset_labels <- -max_height/100
   else offset_labels <- -0.1
   
-  if(rectangle | circular){
+  if(rectangle || circular){
     p <- .ggplot_dend(dend, type = "rectangle", offset_labels = offset_labels, nodes = FALSE,
                       ggtheme = ggtheme, horiz = horiz, circular = circular, palette = palette,
                       labels = show_labels, label_cols = label_cols, 
@@ -175,22 +173,13 @@ fviz_dend <- function(x, k = NULL, h = NULL, k_colors = NULL, palette = NULL,  s
                                  palette = palette, repel = repel,
                                  ggtheme = ggtheme, phylo_layout = phylo_layout, ...)
   }
-#   base plot
-#   else{
-#     plot(dend,  type = type[1], xlab = xlab, ylab = ylab, main = main,
-#          leaflab = leaflab, sub = sub, horiz = horiz,...)
-#     if(rect & !is.null(k))
-#       dendextend::rect.dendrogram(dend, k=k, border = rect_border, 
-#                                   lty = rect_lty, lwd = lwd)
-#   }
-  
   # Add rectangle around clusters
-  if(circular | phylogenic | is.null(k)) rect <- FALSE
-  if(rect_fill & missing(rect_lty)) rect_lty = "blank"
+  if(circular || phylogenic || is.null(k)) rect <- FALSE
+  if(rect_fill && missing(rect_lty)) rect_lty = "blank"
   if(missing(lower_rect)) lower_rect = -(labels_track_height+0.5)
   if(rect){
     p <- p + .rect_dendrogram(dend, k = k, palette = rect_border, rect_fill = rect_fill,
-                              rect_lty = rect_lty, size = lwd, 
+                              rect_lty = rect_lty, linewidth = lwd, 
                               lower_rect = lower_rect)
   }
   
@@ -235,8 +224,7 @@ fviz_dend <- function(x, k = NULL, h = NULL, k_colors = NULL, palette = NULL,  s
   graph_net <- igraph::graph.edgelist(graph_edges)
   
   # extract layout (x-y coords)
-  set.seed(123)
-  graph_layout = layout_func(graph_net)
+  graph_layout <- .with_preserved_seed(123, layout_func(graph_net))
   # number of observations
   nobs <- length(hc$labels)
   
@@ -263,8 +251,10 @@ fviz_dend <- function(x, k = NULL, h = NULL, k_colors = NULL, palette = NULL,  s
   if(!is.null(dendextend::labels_cex(dend))) font.label <- round(dendextend::labels_cex(dend)[1]*12)
   else font.label <- 12
   
+  # FIX: ggplot2 3.0.0+ deprecation - aes_string() replaced with aes() + .data pronoun
+  # See: https://github.com/kassambara/factoextra/issues/190
   p <- ggplot() + geom_segment(data = data.segments,
-                    aes_string(x = "x", y = "y", xend = "xend", yend = "yend"),
+                    aes(x = .data[["x"]], y = .data[["y"]], xend = .data[["xend"]], yend = .data[["yend"]]),
                     lineend = "square")
   if(is.null(label_cols)) label_cols <- "col"
   if(!labels) labels <- NULL
@@ -324,17 +314,26 @@ fviz_dend <- function(x, k = NULL, h = NULL, k_colors = NULL, palette = NULL,  s
   
   p <- ggplot()
   if (segments) {
+    # FIX: ggplot2 3.0.0+ deprecation - aes_string() replaced with aes() + .data pronoun
+    # FIX: ggplot2 3.4.0+ deprecation - size replaced with linewidth for line geoms
+    # See: https://github.com/kassambara/factoextra/issues/190, #191
     p <- p + geom_segment(data = data$segments,
-      aes_string(x = "x", y = "y", xend = "xend", yend = "yend", 
-                 colour = "col", linetype = "lty", size = "lwd"), lineend = "square") +
-      guides(linetype = FALSE, col = FALSE) + #scale_colour_identity() + 
+      aes(x = .data[["x"]], y = .data[["y"]], xend = .data[["xend"]], yend = .data[["yend"]],
+                 colour = .data[["col"]], linetype = .data[["lty"]], linewidth = .data[["lwd"]]), lineend = "square") +
+      # FIX: ggplot2 3.3.4+ deprecation - use "none" instead of FALSE for guides()
+      # See: https://github.com/kassambara/factoextra/issues/179
+      guides(linetype = "none", col = "none") + #scale_colour_identity() +
       scale_size_identity() + scale_linetype_identity()
     if(is.null(palette)) p <- p + scale_colour_identity()
   }
   if (nodes) {
-    p <- p + geom_point(data = data$nodes, 
-      aes_string(x = "x", y = "y", colour = "col", shape = "pch", size = "cex")) + 
-      guides(shape = FALSE, col = FALSE, size = FALSE) + 
+    # FIX: ggplot2 3.0.0+ deprecation - aes_string() replaced with aes() + .data pronoun
+    # See: https://github.com/kassambara/factoextra/issues/190
+    p <- p + geom_point(data = data$nodes,
+      aes(x = .data[["x"]], y = .data[["y"]], colour = .data[["col"]], shape = .data[["pch"]], size = .data[["cex"]])) +
+      # FIX: ggplot2 3.3.4+ deprecation - use "none" instead of FALSE for guides()
+      # See: https://github.com/kassambara/factoextra/issues/179
+      guides(shape = "none", col = "none", size = "none") +
       scale_shape_identity()
   }
   if (labels) {
@@ -346,7 +345,7 @@ fviz_dend <- function(x, k = NULL, h = NULL, k_colors = NULL, palette = NULL,  s
                                 angle = "angle", hjust = "hjust", vjust = "vjust")
   }
   p <- ggpubr::ggpar(p, ggtheme = ggtheme, palette = palette, ...) + theme(axis.line = element_blank())
-  if (horiz & !circular) {
+  if (horiz && !circular) {
     p <- p + coord_flip() + scale_y_reverse()+
       theme(axis.text.y = element_blank(), axis.ticks.y = element_blank(),
             axis.text.x = element_text())
@@ -437,7 +436,7 @@ fviz_dend <- function(x, k = NULL, h = NULL, k_colors = NULL, palette = NULL,  s
 .get_k <- function(dend, k = NULL, h = NULL){
   
   if (!dendextend::is.dendrogram(dend)) stop("x is not a dendrogram object.")
-  if (length(h) > 1L | length(k) > 1L) 
+  if (length(h) > 1L || length(k) > 1L) 
     stop("'k' and 'h' must be a scalar(i.e.: of length 1)")
   tree_heights <- dendextend::heights_per_k.dendrogram(dend)[-1]
   tree_order <- stats::order.dendrogram(dend)
@@ -460,7 +459,7 @@ fviz_dend <- function(x, k = NULL, h = NULL, k_colors = NULL, palette = NULL,  s
           ...) 
 {
   
-  if(missing(k_colors) & !is.null(palette)) k_colors <- palette
+  if(missing(k_colors) && !is.null(palette)) k_colors <- palette
   # value (should be between 0 to 1): proportion of the height 
   # our rect will be between the height needed for k and k+1 clustering.
   prop_k_height <-  0.5
@@ -501,9 +500,12 @@ fviz_dend <- function(x, k = NULL, h = NULL, k_colors = NULL, palette = NULL,  s
   df <- data.frame(xmin = unlist(xleft), ymin = unlist(ybottom), xmax = unlist(xright), ymax = unlist(ytop))
   
   color <- k_colors
-  if(color == "cluster") color <- "default"
-  if(ggpubr:::.is_col_palette(color)) color <- ggpubr:::.get_pal(color, k = k)
-  else if(length(color) > 1 & length(color) < k){
+  # FIX: R 4.2.0+ deprecation - condition has length > 1 warning
+  # Use length() == 1 check before scalar comparison
+  # See: https://github.com/kassambara/factoextra/issues/163, #180
+  if(length(color) == 1 && color == "cluster") color <- "default"
+  if(.is_color_palette(color)) color <- ggpubr::get_palette(color, k = k)
+  else if(length(color) > 1 && length(color) < k){
     color <- rep(color, k)[1:k]
   }
   if(rect_fill){
