@@ -1,6 +1,7 @@
 #' @include eigenvalue.R fviz_add.R
 NULL
 #' @import ggplot2
+#' @importFrom rlang .data
 #' @importFrom ggrepel geom_text_repel
 #' @importFrom grDevices chull
 #' @importFrom graphics plot
@@ -13,7 +14,58 @@ NULL
 #' @importFrom stats rnorm
 #' @importFrom stats runif
 #' @importFrom stats var
-#' 
+#'
+
+# Helper function to check if a palette name is a predefined color palette
+# Replaces ggpubr:::.is_col_palette to avoid using unexported functions
+# Known palette names from RColorBrewer, ggsci, and ggpubr defaults
+.is_color_palette <- function(pal) {
+ if (is.null(pal)) return(FALSE)
+ # RColorBrewer palettes
+ brewer_pals <- c("Blues", "BuGn", "BuPu", "GnBu", "Greens", "Greys", "Oranges",
+                  "OrRd", "PuBu", "PuBuGn", "PuRd", "Purples", "RdPu", "Reds",
+                  "YlGn", "YlGnBu", "YlOrBr", "YlOrRd", "BrBG", "PiYG", "PRGn",
+                  "PuOr", "RdBu", "RdGy", "RdYlBu", "RdYlGn", "Spectral",
+                  "Accent", "Dark2", "Paired", "Pastel1", "Pastel2", "Set1",
+                  "Set2", "Set3")
+ # ggsci palettes
+ ggsci_pals <- c("npg", "aaas", "nejm", "lancet", "jama", "jco", "ucscgb",
+                 "d3", "locuszoom", "igv", "uchicago", "startrek", "tron",
+                 "futurama", "rickandmorty", "simpsons")
+ # ggpubr built-in
+ other_pals <- c("default", "hue", "grey_pal", "gray_pal")
+ all_pals <- c(brewer_pals, ggsci_pals, other_pals)
+ return(length(pal) == 1 && pal[1] %in% all_pals)
+}
+
+.class_label <- function(x){
+  paste(class(x), collapse = ", ")
+}
+
+.extract_nested_element <- function(x, element){
+  parts <- strsplit(element, "$", fixed = TRUE)[[1]]
+  out <- x
+  for(part in parts){
+    if(is.null(out)) return(NULL)
+    out <- out[[part]]
+  }
+  out
+}
+
+.with_preserved_seed <- function(seed, expr){
+  has_seed <- exists(".Random.seed", envir = .GlobalEnv, inherits = FALSE)
+  if(has_seed) old_seed <- get(".Random.seed", envir = .GlobalEnv)
+  on.exit({
+    if(!has_seed && exists(".Random.seed", envir = .GlobalEnv, inherits = FALSE)){
+      rm(".Random.seed", envir = .GlobalEnv)
+    } else if(has_seed){
+      assign(".Random.seed", old_seed, envir = .GlobalEnv)
+    }
+  }, add = TRUE)
+  set.seed(seed)
+  force(expr)
+}
+
 # Check and get the class of the output of a factor analysis
 # ++++++++++++++++++++++++++++
 # X: an output of factor analysis (PCA, CA, MCA, MFA) 
@@ -22,7 +74,7 @@ NULL
   
   if(inherits(X, c('PCA', 'princomp', 'prcomp')))
     facto_class ="PCA"
-  else if(inherits(X, 'pca') & inherits(X, 'dudi'))
+  else if(inherits(X, "pca") && inherits(X, "dudi"))
     facto_class ="PCA"
   else if(inherits(X, c("CA", "ca", "coa", "correspondence"))) facto_class="CA"
   else if(inherits(X, c("MCA", "acm"))) facto_class = "MCA"
@@ -34,7 +86,7 @@ NULL
    else if (inherits(X$ExPosition.Data,'epPCA')) facto_class="PCA"
    else if (inherits(X$ExPosition.Data,'epMCA')) facto_class="MCA"
   }
-  else stop("An object of class : ", class(X), 
+  else stop("An object of class : ", .class_label(X), 
             " can't be handled by factoextra")   
 }
 
@@ -51,17 +103,16 @@ NULL
 .get_supp <- function(X, element = NULL, axes = 1:2, 
                       result = c("coord", "cos2"), select = NULL){
   
-  if(inherits(X, "MFA") & element == "group")
+  if(inherits(X, "MFA") && element == "group")
     elmt <- .get_mfa_group_sup(X)
   else if(inherits(X, c("CA", "PCA", "MCA", "MFA", "HMFA", "FAMD"))) {
-    exprs <- paste0("X$", element)
-    elmt <- eval(parse(text=exprs ))
+    elmt <- .extract_nested_element(X, element)
   }
   else if(inherits(X, "ca")){
     if(element == "col.sup") elmt <- .get_ca_col_sup(X)
     else if(element == "row.sup") elmt <- .get_ca_row_sup(X)
   }
-  else stop("An object of class : ", class(X), 
+  else stop("An object of class : ", .class_label(X), 
             " can't be handled by the function .get_supp()")
   
   # summarize the result
@@ -81,14 +132,14 @@ NULL
     # 1.Extract the coordinates x, y and coord
     if("coord" %in% result){
       dd <- data.frame(elmt$coord[, axes, drop=FALSE])
-      coord <- apply(dd^2, 1, sum) # x^2 + y2 + ...
+      coord <- rowSums(dd^2, na.rm = TRUE) # x^2 + y2 + ...
       res <- cbind(dd, coord = coord)
     }
     
     # 2. Extract the cos2
     if("cos2" %in% result){
       cos2 <- data.frame(elmt$cos2[, axes, drop=FALSE])
-      if(length(axes) > 1) cos2 <- apply(cos2, 1, sum, na.rm=TRUE)
+      if(length(axes) > 1) cos2 <- rowSums(cos2, na.rm = TRUE)
       res <- cbind(res, cos2 = cos2)
     }
     
@@ -132,31 +183,39 @@ NULL
 }
 
 # get supplementary columns from ca output (ca package)
+#
+# OPTIMIZATION: Replaced t(apply()) with sweep() for column-wise multiplication
+# sweep() is much faster for element-wise row/column operations
 .get_ca_col_sup <- function(res.ca){
   # supplementary points
   index <- res.ca$colsup
-  cols <- NULL 
+  cols <- NULL
   if(length(index) > 0){
-    # principal coord = standard coord X sqrt(eig)
-    coord <- t(apply(res.ca$colcoord, 1, "*", res.ca$sv))
-    cos2 <- apply(coord^2, 2, "/", res.ca$coldist^2)
-    cols <- list(coord = coord[index, , drop = FALSE], 
-                cos2 = cos2[index, , drop = FALSE]) 
+    # OPTIMIZED: principal coord = standard coord X sqrt(eig)
+    # Using sweep() instead of t(apply()) - multiply each column by sv
+    coord <- sweep(res.ca$colcoord, 2, res.ca$sv, "*")
+    # OPTIMIZED: cos2 = coord^2 / dist^2 (divide each row by its distance squared)
+    cos2 <- coord^2 / res.ca$coldist^2
+    cols <- list(coord = coord[index, , drop = FALSE],
+                cos2 = cos2[index, , drop = FALSE])
   }
   cols
 }
 
 # get supplementary rows from ca output (ca package)
+#
+# OPTIMIZATION: Replaced t(apply()) with sweep() for column-wise multiplication
 .get_ca_row_sup <- function(res.ca){
   rows <- NULL
   # supplementary points
   index <- res.ca$rowsup
   if(length(index) > 0){
-    # principal coord = standard coord X sqrt(eig)
-    coord <- t(apply(res.ca$rowcoord, 1, "*", res.ca$sv))
-    cos2 <- apply(coord^2, 2, "/", res.ca$rowdist^2)
-    rows <- list(coord = coord[index, , drop = FALSE], 
-                                     cos2 = cos2[index, , drop = FALSE]) 
+    # OPTIMIZED: principal coord = standard coord X sqrt(eig)
+    coord <- sweep(res.ca$rowcoord, 2, res.ca$sv, "*")
+    # OPTIMIZED: cos2 = coord^2 / dist^2
+    cos2 <- coord^2 / res.ca$rowdist^2
+    rows <- list(coord = coord[index, , drop = FALSE],
+                                     cos2 = cos2[index, , drop = FALSE])
   }
   rows
 }
@@ -328,7 +387,7 @@ NULL
   sv <- sqrt(eig)
   
   # rows/columns in principal coordinates
-  if(type %in% c("symmetric", "colprincipal", "colgab", "cogreen")) res <- data
+  if(type %in% c("symmetric", "colprincipal", "colgab", "colgreen")) res <- data
   else{
     # get standard coordinates 
     x <- res$x/sqrt(eig[1])
@@ -383,8 +442,8 @@ NULL
   }
   # Mass package
   else if(inherits(res.ca, "correspondence")){
-    row.sum <- apply(res.ca$Freq, 1, sum)
-    col.sum <- apply(res.ca$Freq, 2, sum)
+    row.sum <- rowSums(res.ca$Freq)
+    col.sum <- colSums(res.ca$Freq)
     n <- sum(res.ca$Freq)
     if(element =="row") mass <- row.sum/n
     else if(element =="col") mass <- col.sum/n
@@ -394,7 +453,7 @@ NULL
     if(element =="row") mass <- res.ca$ExPosition.Data$M
     else if(element =="col") mass <- res.ca$ExPosition.Data$W
   }
-  else stop("An object of class : ", class(res.ca), 
+  else stop("An object of class : ", .class_label(res.ca), 
             " can't be handled") 
   return(mass)
 }
@@ -407,7 +466,7 @@ NULL
     centered <- is.numeric(res.pca$center)
     scaled <- is.numeric(res.pca$scale)
     
-    if(centered & scaled) 
+    if(centered && scaled) 
       t(t(res.pca$x %*% t(res.pca$rotation)) * res.pca$scale + res.pca$center)
     else if(centered)
       t(t(res.pca$x %*% t(res.pca$rotation)) + res.pca$center)
@@ -424,7 +483,7 @@ NULL
               "scores = TRUE.")
   }
   
-  else stop("Can't handle an object of class ", class(res.pca))
+  else stop("Can't handle an object of class ", .class_label(res.pca))
 }
 
 
@@ -480,18 +539,16 @@ NULL
       name <- filter$name
       common <- intersect(name, d$name)
       diff <- setdiff(name, d$name)
-      #if(check & length(common) == 0) stop("Can't find the specified names")
-      # if(check & length(diff)!=0) warning("Can't find the the following name(s): ", diff)
       d <- d[common, , drop = FALSE]
     }
     
     # Filter by cos2
-    if(!is.null(filter$cos2) & nrow(d) >= 1){
+    if(!is.null(filter$cos2) && nrow(d) >= 1){
       # case 1 cos2 is in [0, 1]
       # rows with cos2 > value are selected
-      if(0 <= filter$cos2 & filter$cos2 <= 1){
+      if(0 <= filter$cos2 && filter$cos2 <= 1){
         d <- d[which(d$cos2 >= filter$cos2), , drop = FALSE]
-        if(check & nrow(d)==0)
+        if(check && nrow(d)==0)
           stop("There are no observations with cos2 >=", filter$cos2, 
                ". Please, change the value of cos2 and try again.")
       }
@@ -499,16 +556,16 @@ NULL
       else if(filter$cos2 > 1){
         cos2 <- round(filter$cos2)
         d <- d[order(d$cos2, decreasing = TRUE), , drop = FALSE]
-        d <- d[1:min(filter$cos2, nrow(d)),, drop = FALSE]
+        d <- d[seq_len(min(filter$cos2, nrow(d))),, drop = FALSE]
       }
     }
     
     # Filter by contrib: the top rows are selected 
-    if(!is.null(filter$contrib) & nrow(d) >= 1){
+    if(!is.null(filter$contrib) && nrow(d) >= 1){
       contrib <- round(filter$contrib)
       if(contrib < 1) stop("The value of the argument contrib >", 1)
       d <- d[order(d$contrib, decreasing = TRUE), , drop = FALSE]
-      d <- d[1:min(contrib, nrow(d)), , drop = FALSE]
+      d <- d[seq_len(min(contrib, nrow(d))), , drop = FALSE]
     }
     
   }
@@ -534,19 +591,22 @@ NULL
   
  
   # top elements
-  if(top!=Inf & top < length(x))
+  if(top != Inf && top < length(x))
     x <- sort(x, decreasing=TRUE)[1:top]
   # sorting
   if(sort.value[1]=="desc") x <- sort(x, decreasing = TRUE)
   else if(sort.value[1]=="asc") x <- sort(x, decreasing = FALSE)
   # bar names
-  if(is.null(names(x))) names(x) <- 1:length(x)
+  if(is.null(names(x))) names(x) <- seq_along(x)
   
   #data frame for ggplot2
   d <- cbind.data.frame(name = factor(names(x), levels = names(x)), val = x)
   
   # plot
-  p <- ggplot(d, aes_string("name", "val")) + 
+  # FIX: ggplot2 3.0.0+ deprecation - aes_string() replaced with aes() + .data pronoun
+  # aes_string() was soft-deprecated in ggplot2 3.0.0
+  # See: https://github.com/kassambara/factoextra/issues/190
+  p <- ggplot(d, aes(x = .data[["name"]], y = .data[["val"]])) +
     geom_bar(stat="identity", fill=fill, color = color) +
     labs(title = title,  x =xlab, y = ylab)+
     theme(axis.text.x = element_text(angle=45), 
@@ -566,15 +626,13 @@ NULL
   if(!is.null(data)){
     if(!is.null(jitter$width)){
       width <- abs(jitter$width)
-     set.seed(1234)
-      xjit <- runif(nrow(data), min = -width, max = width)
+      xjit <- .with_preserved_seed(1234, runif(nrow(data), min = -width, max = width))
       data$x <- data$x + ((xjit + rev(xjit))/2)
     }
     
     if(!is.null(jitter$height)){
       height <- abs(jitter$height)
-       set.seed(12345)
-      yjit <- runif(nrow(data), min = -height, max = height)
+      yjit <- .with_preserved_seed(12345, runif(nrow(data), min = -height, max = height))
       data$y <- data$y + ((yjit + rev(yjit))/2)
     }
     
@@ -624,7 +682,7 @@ NULL
                "col", "col.sup" # col - ca
                )
   for(el in element){
-    if(label[1] == "all" | el %in% label) lab[[el]] <- TRUE
+    if(label[1] == "all" || el %in% label) lab[[el]] <- TRUE
     else lab[[el]] <- FALSE
   }
   lab
@@ -657,36 +715,36 @@ NULL
 # Generate a data containing a cluster of any shapes
 # For comparison between dbscan and k-means
 .generate_multishapes <- function(){
-  # First circle (big)
-  set.seed(1234)
-  x <- matrix(rnorm(800, sd = 2), ncol=2)
-  y <- x/sqrt(rowSums(x^2))
-  y[,1] <- y[,1] + rnorm(400, 0, 0.1)
-  y[,2] <- y[,2] + rnorm(400, 0, 0.1)
-  x1 <- y[, 1]
-  y1 <- y[, 2]
-  # Second circle (small)
-  x2 <- x1/2.5
-  y2 <- y1/2.5
-  shape <- rep(c(1,2), each = 400)
-  # Line 1
-  x3 <- runif(100, min = -1.5, 0)
-  y3 <- rnorm(100, -2, 0.1)
-  shape <- c(shape, rep(3, 100))
-  # Line 2
-  x4 <- runif(100, min = -1.5, 0)
-  y4 <- rnorm(100, -3, 0.1)
-  shape <- c(shape, rep(4, 100))
-  # compact points
-  x6 <- rnorm(50, 1, 0.1)
-  y6 <- rnorm(50, -2.5, 0.1)
-  shape <- c(shape, rep(5, 50))
-  # noises/outliers
-  x5 <- runif(50, min = -1.5, 1.5)
-  y5 <- rnorm(50, -1, 1)
-  shape <- c(shape, rep(6, 50))
-  multishapes <- data.frame(x = c(x1, x2, x3, x4, x5, x6), y = c(y1, y2, y3, y4, y5, y6), shape = shape)
-  multishapes
+  .with_preserved_seed(1234, {
+    # First circle (big)
+    x <- matrix(rnorm(800, sd = 2), ncol = 2)
+    y <- x/sqrt(rowSums(x^2))
+    y[,1] <- y[,1] + rnorm(400, 0, 0.1)
+    y[,2] <- y[,2] + rnorm(400, 0, 0.1)
+    x1 <- y[, 1]
+    y1 <- y[, 2]
+    # Second circle (small)
+    x2 <- x1/2.5
+    y2 <- y1/2.5
+    shape <- rep(c(1,2), each = 400)
+    # Line 1
+    x3 <- runif(100, min = -1.5, 0)
+    y3 <- rnorm(100, -2, 0.1)
+    shape <- c(shape, rep(3, 100))
+    # Line 2
+    x4 <- runif(100, min = -1.5, 0)
+    y4 <- rnorm(100, -3, 0.1)
+    shape <- c(shape, rep(4, 100))
+    # compact points
+    x6 <- rnorm(50, 1, 0.1)
+    y6 <- rnorm(50, -2.5, 0.1)
+    shape <- c(shape, rep(5, 50))
+    # noises/outliers
+    x5 <- runif(50, min = -1.5, 1.5)
+    y5 <- rnorm(50, -1, 1)
+    shape <- c(shape, rep(6, 50))
+    data.frame(x = c(x1, x2, x3, x4, x5, x6), y = c(y1, y2, y3, y4, y5, y6), shape = shape)
+  })
 }
 
 
@@ -707,9 +765,23 @@ NULL
 # ind: individuals data generated by facto_summarize()
 # grp: group column index or factor
 .add_ind_groups <- function(X, ind, grp){
- 
-  if(inherits(X, c("PCA", "MCA", "MFA", "FAMD")) & length(grp) > 1){
-    if(is.numeric(grp) | is.character(grp)) grp <- as.data.frame(X$call$X[rownames(ind), grp, drop = FALSE])
+  if(inherits(X, c("PCA", "MCA", "MFA", "FAMD")) && length(grp) > 1){
+    if(is.numeric(grp) || is.character(grp)){
+      data <- X$call$X
+      # Only treat grp as column selectors when it isn't a per-row grouping vector.
+      if(length(grp) != nrow(ind)){
+        if(is.numeric(grp)){
+          if(!all(grp %in% seq_len(ncol(data)))){
+            stop("Some grouping variable indices are out of range for the original data.")
+          }
+        } else {
+          if(!all(grp %in% colnames(data))){
+            stop("Some grouping variable names are not present in the original data.")
+          }
+        }
+        grp <- as.data.frame(data[rownames(ind), grp, drop = FALSE])
+      }
+    }
     #if(!is.null(X$call$ind.sup)) grp <- grp[-X$call$ind.sup, , drop = FALSE]
   }
   habillage <- grp
@@ -718,16 +790,24 @@ NULL
     if(nrow(ind) != nrow(grp)) stop("The length of grouping variables ",
                                     "should be the same as the number of individuals.")
     ind <- cbind.data.frame(ind, grp)
-    ind[, colnames(grp)] <- apply(ind[, colnames(grp)], 2, as.character)
-    ind <- tidyr::gather_(ind, key_col = "facet_vars", value_col = "Groups",
-                          gather_cols = colnames(grp))
+    ind[, colnames(grp)] <- apply(ind[, colnames(grp), drop = FALSE], 2, as.character)
+    # Convert wide to long format using base R (replaces tidyr::pivot_longer)
+    grp_cols <- colnames(grp)
+    id_cols <- setdiff(colnames(ind), grp_cols)
+    ind <- stats::reshape(ind, direction = "long",
+                          varying = grp_cols,
+                          v.names = "Groups",
+                          timevar = "facet_vars",
+                          times = grp_cols,
+                          idvar = id_cols)
+    rownames(ind) <- NULL
     ind$facet_vars <- as.factor(ind$facet_vars)
     ind$Groups <- as.factor(ind$Groups)
     name.quali <- "Groups"
   }
   else{
     # X is from FactoMineR outputs
-    if(inherits(X, c("PCA", "MCA", "MFA", "FAMD")) & length(habillage) == 1){
+    if(inherits(X, c("PCA", "MCA", "MFA", "FAMD")) && length(habillage) == 1){
       data <- X$call$X
       if (is.numeric(habillage)) name.quali <- colnames(data)[habillage]
       else name.quali <- habillage 
@@ -750,40 +830,318 @@ NULL
 }
 
 
+# MFA helpers
+.mfa_group_names <- function(X){
+  if(!inherits(X, "MFA")) return(NULL)
+  group.names <- NULL
+  if(!is.null(X$group$Lg)){
+    group.names <- rownames(X$group$Lg)
+    if(!is.null(group.names) && length(group.names) > 0){
+      if(length(group.names) == length(X$call$group) + 1)
+        group.names <- group.names[-length(group.names)]
+      else if(length(group.names) > length(X$call$group))
+        group.names <- group.names[seq_along(X$call$group)]
+    }
+  }
+  if(is.null(group.names) || length(group.names) == 0){
+    if(!is.null(X$call$name.group)) group.names <- X$call$name.group
+  }
+  if(is.null(group.names) || length(group.names) == 0){
+    group.names <- paste0("Gr", seq_along(X$call$group))
+  }
+  group.names
+}
+
+.mfa_group_map <- function(X){
+  if(!inherits(X, "MFA")) return(NULL)
+  data <- X$call$X
+  if(is.null(data)) return(NULL)
+  data <- as.data.frame(data)
+  group.sizes <- X$call$group
+  group.types <- X$call$type
+  if(is.null(group.sizes) || is.null(group.types)) return(NULL)
+
+  group.names <- .mfa_group_names(X)
+  vars <- colnames(data)
+  if(is.null(vars)) vars <- paste0("V", seq_len(ncol(data)))
+
+  group.id <- rep(seq_along(group.sizes), group.sizes)
+  if(length(group.id) != length(vars)){
+    n <- min(length(group.id), length(vars))
+    group.id <- group.id[seq_len(n)]
+    vars <- vars[seq_len(n)]
+  }
+
+  is.group.sup <- rep(FALSE, length(group.sizes))
+  if(!is.null(X$call$num.group.sup)) is.group.sup[X$call$num.group.sup] <- TRUE
+  var.is.numeric <- vapply(data[, seq_along(vars), drop = FALSE], is.numeric, logical(1))
+
+  var.type <- character(length(vars))
+  for(i in seq_along(vars)){
+    g <- group.id[i]
+    gtype <- group.types[g]
+    if(gtype %in% c("c", "s")) var.type[i] <- "quanti"
+    else if(gtype == "n") var.type[i] <- "quali"
+    else if(gtype == "m") var.type[i] <- ifelse(var.is.numeric[i], "quanti", "quali")
+    else var.type[i] <- NA_character_
+  }
+
+  data.frame(
+    var = vars,
+    group = group.names[group.id],
+    group.id = group.id,
+    group.type = group.types[group.id],
+    var.type = var.type,
+    is.group.sup = is.group.sup[group.id],
+    stringsAsFactors = FALSE
+  )
+}
+
 # MFA: get quantitative variables groups
 # For plotting
 .get_quanti_var_groups <- function(X){
-  group <- data.frame(name = rownames(X$group$Lg[-nrow(X$group$Lg),,drop=FALSE]),
-                      nvar = X$call$group, type = X$call$type)
-  is.group.sup <- !is.null(X$call$num.group.sup)
-  if(is.group.sup) group <- group[-X$call$num.group.sup, , drop = FALSE]
-  group <- subset(group, group$type == "c")
-  group <- rep(group$name, group$nvar)
-  group
+  map <- .mfa_group_map(X)
+  if(is.null(map)) return(NULL)
+  quanti <- map[map$var.type == "quanti" & !map$is.group.sup, , drop = FALSE]
+  if(nrow(quanti) == 0) return(NULL)
+  if(!is.null(X$quanti.var$coord)){
+    qnames <- rownames(X$quanti.var$coord)
+    idx <- match(qnames, quanti$var)
+    if(any(is.na(idx))) return(quanti$group)
+    return(quanti$group[idx])
+  }
+  quanti$group
 }
 
 # Get qualitative supplementary variables
-# Each variables is repeated xtimes = levels(variables)
+# Each variable is repeated xtimes = levels(variables)
 # Used to color variable categories by variables
 .get_quali_var_sup_names <- function(X){
-  group <- data.frame(name = rownames(X$group$Lg[-nrow(X$group$Lg),,drop=FALSE]),
-                      nvar = X$call$group, type = X$call$type)
-  is.group.sup <- !is.null(X$call$num.group.sup)
+  map <- .mfa_group_map(X)
+  if(is.null(map)) return(NULL)
+  sup <- map[map$is.group.sup & map$var.type == "quali", , drop = FALSE]
+  if(nrow(sup) == 0) return(NULL)
+  data <- X$call$X
   res <- NULL
-  if(is.group.sup){
-    # Get group sup names
-    group.sup <- group[X$call$num.group.sup, , drop = FALSE]
-    group.sup <- subset(group.sup, group.sup$type == "n", drop = FALSE)
-    group.sup.name <- as.character(group.sup$name)
-    # Names of variables in the data
-    data <- X$call$X
-    vars <- colnames(data)
-    vars.groups <- rep(group$name, group$nvar)
-    group.sup.vars <- vars[vars.groups %in% group.sup.name]
-    for(v in group.sup.vars){
-      res <- c(res, rep(v, length(levels(data[,v]))))
+  for(v in sup$var){
+    res <- c(res, rep(v, length(levels(as.factor(data[, v])))))
+  }
+  res
+}
+
+.factominer_needs_category_map <- function(facto.class, element){
+  if(!(facto.class %in% c("MCA", "MFA", "FAMD", "HMFA"))) return(FALSE)
+  if(element == "var") return(facto.class == "MCA")
+  element %in% c("quali.var", "quali.sup")
+}
+
+.split_partial_names <- function(names, group.names = NULL){
+  if(length(names) == 0) return(data.frame(name = character(0), group.name = character(0)))
+  name.part <- rep(NA_character_, length(names))
+  group.part <- rep(NA_character_, length(names))
+
+  if(!is.null(group.names) && length(group.names) > 0){
+    escape_regex <- function(x){
+      if(is.na(x)) return(NA_character_)
+      specials <- c("\\", ".", "^", "$", "|", "(", ")", "[", "]", "{", "}", "*", "+", "?")
+      for(s in specials){
+        x <- gsub(s, paste0("\\\\", s), x, fixed = TRUE)
+      }
+      x
+    }
+    escaped <- vapply(group.names, escape_regex, character(1))
+    pattern <- paste0("\\.(", paste(escaped, collapse = "|"), ")$")
+    matched <- grepl(pattern, names)
+    if(any(matched)){
+      group.part[matched] <- sub(pattern, "\\1", names[matched])
+      name.part[matched] <- sub(pattern, "", names[matched])
+    }
+  } else {
+    matched <- rep(FALSE, length(names))
+  }
+
+  fallback <- !matched
+  if(any(fallback)){
+    split.pos <- regexpr(".", names[fallback], fixed = TRUE)
+    name.part[fallback] <- ifelse(split.pos > 0, substr(names[fallback], 1, split.pos - 1), names[fallback])
+    group.part[fallback] <- ifelse(split.pos > 0, substr(names[fallback], split.pos + 1, nchar(names[fallback])), NA_character_)
+  }
+
+  data.frame(name = name.part, group.name = group.part)
+}
+
+#' Map FactoMineR category labels to legacy naming patterns
+#'
+#' @param X a FactoMineR object (MCA, MFA, FAMD, HMFA).
+#' @param element element to map. Use "var" for MCA categories or "quali.var"
+#'   for MFA/FAMD/HMFA qualitative categories. "quali.sup" maps supplementary
+#'   qualitative categories when available.
+#' @return A data.frame with current labels, variable names, levels, and legacy
+#'   naming patterns.
+#' @examples
+#' \donttest{
+#' if (requireNamespace("FactoMineR", quietly = TRUE)) {
+#'   data(poison)
+#'   res.mca <- FactoMineR::MCA(poison, quanti.sup = 1:2, quali.sup = 3:4, graph = FALSE)
+#'   head(factominer_category_map(res.mca, element = "var"))
+#' }
+#' }
+#' @export
+factominer_category_map <- function(X, element = c("quali.var", "quali.sup", "var")){
+  element <- match.arg(element)
+  facto.class <- .get_facto_class(X)
+  if(!facto.class %in% c("MCA", "MFA", "FAMD", "HMFA"))
+    stop("factominer_category_map() supports MCA, MFA, FAMD, and HMFA objects only.")
+
+  elmt <- NULL
+  if(element == "var"){
+    if(!inherits(X, "MCA")) stop("element = 'var' is only supported for MCA outputs.")
+    elmt <- X$var
+  } else if(element == "quali.var"){
+    if(!is.null(X$quali.var)) elmt <- X$quali.var
+    else if(inherits(X, "MCA")) elmt <- X$var
+  } else if(element == "quali.sup"){
+    elmt <- X$quali.sup
+  }
+
+  if(is.null(elmt) || is.null(elmt$coord)){
+    warning("No qualitative categories found for element = '", element, "'.")
+    return(data.frame(current = character(0), variable = character(0), level = character(0),
+                      legacy_dot = character(0), legacy_underscore = character(0),
+                      legacy_equals = character(0), legacy_colon = character(0),
+                      legacy_level = character(0), stringsAsFactors = FALSE))
+  }
+
+  current.names <- rownames(elmt$coord)
+  if(is.null(current.names)) current.names <- character(0)
+
+  data <- X$call$X
+  if(is.null(data)){
+    warning("Original data not found in X$call$X; cannot build category map.")
+    return(data.frame(current = current.names, variable = NA_character_, level = NA_character_,
+                      legacy_dot = NA_character_, legacy_underscore = NA_character_,
+                      legacy_equals = NA_character_, legacy_colon = NA_character_,
+                      legacy_level = NA_character_, stringsAsFactors = FALSE))
+  }
+  data <- as.data.frame(data)
+  is.quali <- which(!vapply(data, is.numeric, logical(1)))
+  if(length(is.quali) == 0){
+    return(data.frame(current = current.names, variable = NA_character_, level = NA_character_,
+                      legacy_dot = NA_character_, legacy_underscore = NA_character_,
+                      legacy_equals = NA_character_, legacy_colon = NA_character_,
+                      legacy_level = NA_character_, stringsAsFactors = FALSE))
+  }
+
+  data.quali <- data[, is.quali, drop = FALSE]
+  data.quali <- droplevels(as.data.frame(lapply(data.quali, as.factor)))
+  level.orig <- lapply(data.quali, levels)
+
+  niveau <- unlist(level.orig)
+  if(sum(duplicated(niveau)) > 0){
+    for(j in seq_along(data.quali)){
+      if (sum(niveau %in% levels(data.quali[[j]])) != nlevels(data.quali[[j]])) {
+        levels(data.quali[[j]]) <- paste(names(data.quali)[j], levels(data.quali[[j]]), sep = "_")
+      }
     }
   }
+
+  variable <- rep(names(data.quali), vapply(data.quali, nlevels, integer(1)))
+  listModa <- unlist(lapply(data.quali, levels))
+  wlistModa <- which(listModa %in% c("y", "n", "Y", "N"))
+  if(length(wlistModa) > 0){
+    listModa[wlistModa] <- paste(variable[wlistModa], listModa[wlistModa], sep = ".")
+  }
+  level <- unlist(level.orig)
+
+  map.all <- data.frame(current = listModa, variable = variable, level = level, stringsAsFactors = FALSE)
+  map <- data.frame(current = current.names, variable = NA_character_, level = NA_character_, stringsAsFactors = FALSE)
+  if(length(current.names) > 0){
+    idx <- match(current.names, map.all$current)
+    matched <- !is.na(idx)
+    if(any(matched)){
+      map[matched, c("variable", "level")] <- map.all[idx[matched], c("variable", "level")]
+    }
+  }
+
+  map$legacy_dot <- ifelse(is.na(map$variable), NA_character_, paste(map$variable, map$level, sep = "."))
+  map$legacy_underscore <- ifelse(is.na(map$variable), NA_character_, paste(map$variable, map$level, sep = "_"))
+  map$legacy_equals <- ifelse(is.na(map$variable), NA_character_, paste(map$variable, map$level, sep = "="))
+  map$legacy_colon <- ifelse(is.na(map$variable), NA_character_, paste(map$variable, map$level, sep = ":"))
+  map$legacy_level <- map$level
+  map
+}
+
+#' Map legacy FactoMineR category names to current labels
+#'
+#' @param X a FactoMineR object (MCA, MFA, FAMD, HMFA).
+#' @param names character vector of category labels.
+#' @param element element to map. Use "var" for MCA categories or "quali.var"
+#'   for MFA/FAMD/HMFA qualitative categories. "quali.sup" maps supplementary
+#'   qualitative categories when available.
+#' @param quiet if TRUE, suppress warnings.
+#' @return Character vector of mapped labels.
+#' @examples
+#' \donttest{
+#' if (requireNamespace("FactoMineR", quietly = TRUE)) {
+#'   data(poison)
+#'   res.mca <- FactoMineR::MCA(poison, quanti.sup = 1:2, quali.sup = 3:4, graph = FALSE)
+#'   map <- factominer_category_map(res.mca, element = "var")
+#'   map_factominer_legacy_names(res.mca, map$legacy_underscore[1:3], element = "var")
+#' }
+#' }
+#' @export
+map_factominer_legacy_names <- function(X, names, element = c("quali.var", "quali.sup", "var"), quiet = FALSE){
+  element <- match.arg(element)
+  names <- as.character(names)
+  map <- factominer_category_map(X, element = element)
+  if(nrow(map) == 0) return(names)
+
+  candidate.cols <- c("current", "legacy_dot", "legacy_underscore", "legacy_equals", "legacy_colon", "legacy_level")
+  candidate.tbl <- do.call(rbind, lapply(candidate.cols, function(col){
+    data.frame(candidate = map[[col]], current = map$current, stringsAsFactors = FALSE)
+  }))
+  candidate.tbl <- candidate.tbl[!is.na(candidate.tbl$candidate) & nzchar(candidate.tbl$candidate), , drop = FALSE]
+  if(nrow(candidate.tbl) == 0) return(names)
+
+  counts <- table(candidate.tbl$candidate)
+  ambiguous <- names(counts[counts > 1])
+  lookup <- candidate.tbl[!candidate.tbl$candidate %in% ambiguous, , drop = FALSE]
+  lookup <- stats::setNames(lookup$current, lookup$candidate)
+
+  res <- names
+  mapped <- logical(length(names))
+  unmatched <- character(0)
+
+  for(i in seq_along(names)){
+    nm <- names[i]
+    if(is.na(nm) || !nzchar(nm)) next
+    if(nm %in% map$current) next
+    if(nm %in% names(lookup)){
+      res[i] <- lookup[[nm]]
+      mapped[i] <- TRUE
+    } else {
+      unmatched <- c(unmatched, nm)
+    }
+  }
+
+  if(!quiet){
+    if(any(mapped)){
+      warning(
+        "Mapped legacy FactoMineR category labels to current names. ",
+        "Use factominer_category_map() to inspect mappings."
+      )
+    }
+    if(length(unmatched) > 0){
+      unmatched <- unique(unmatched)
+      warning(
+        "Some category labels were not found in the current FactoMineR output: ",
+        paste(utils::head(unmatched, 8), collapse = ", "),
+        if(length(unmatched) > 8) " ..." else "",
+        ". Use factominer_category_map() and map_factominer_legacy_names() to map legacy labels."
+      )
+    }
+  }
+
   res
 }
 
@@ -792,4 +1150,3 @@ NULL
 f_pca <- function(X, graph = FALSE){
   FactoMineR::PCA(X, graph = FALSE)
 }
-
